/* $Id: debug.c,v 1.5 2002/11/25 12:02:40 ls Exp $ */

/**
 * \file debug.c
 * \brief Functions used to generate debugging output.
 * \author Juerg Reusser, Luzian Scherrer
 *
 * This module provides methods to display the debugging messages
 * generated by all the other parts of the sourcecode. It acts according
 * the the defined debug-level.
 */

#include <stdio.h>
#include "globals.h"
#include "debug.h"

/**
 *  Print debug messages to STDOUT if the parameter level contains the 
 *  current DEBUG_LEVEL.
 *
 *  @param level The debug-level the caller is sending a message for.
 *  @param message Pointer to the text of the current message.
 */
void debug(int level, char *message)
{
  if((level)&(DEBUG_LEVEL)) 
  {
    switch(level)
    {
      case DEBUG_PARSER:
        printf("DEBUG: [DEBUG_PARSER] %s\n", message);
        break;
      case DEBUG_CODEGEN:
        printf("DEBUG: [DEBUG_CODEGEN] %s\n", message);
        break;
      case DEBUG_EXECUTION:
        printf("DEBUG: [DEBUG_EXECUTION] %s\n", message);
        break;
      case DEBUG_DUMPREGS:
        printf("DEBUG: [DEBUG_DUMPREGS] %s\n", message);
        break;
      case DEBUG_HOSTLIST:
        printf("DEBUG: [DEBUG_HOSTLIST] %s\n", message);
        break;
      case DEBUG_ICMP:
        printf("DEBUG: [DEBUG_ICMP] %s\n", message);
        break;
      case DEBUG_TRACE:
        printf("%s\n", message);
        break;
    }
    fflush(stdout);
  }
}
