/* $Id: precalc.h,v 1.2 2002/11/29 13:47:46 ls Exp $ */

/**
 * \file precalc.h
 * \brief Precalculated sequences and checksums
 * \author Juerg Reusser, Luzian Scherrer
 *
 * This header contains the precalculated sequence-numbers and their 
 * corresponding base-checksums used for building the candidate ICMP
 * packets.
 */

/*
 * The following PERL program precalculates both arrays:
 *.......................................................................

#!/usr/bin/perl

$type = 0x0800;
$id   = 0xa800;

print "int checksum[] = { ";
for($seq = 0x0100, $c = 1; $seq < 0xff00; $seq += 0x0100, $c++) {
  printf("0x%04x, ", ~($type + $id + $seq) & 0xffff);
  if($c%7==0) { print "\n                   "; }
}
printf("0x%04x };\n\n",  ~($type + $id + $seq) & 0xffff);

print "int sequence[] = { ";
for($seq = 0x0100, $c = 1; $seq < 0xff00; $seq += 0x0100, $c++) {
  printf("0x%04x, ", $seq);
  if($c%7==0) { print "\n                   "; }
}
printf("0x%04x };\n\n",  $seq);

 *.......................................................................
 */


/** Precalculated ICMP checksums */
int checksum[] = { 0x4eff, 0x4dff, 0x4cff, 0x4bff, 0x4aff, 0x49ff, 0x48ff, 
                   0x47ff, 0x46ff, 0x45ff, 0x44ff, 0x43ff, 0x42ff, 0x41ff, 
                   0x40ff, 0x3fff, 0x3eff, 0x3dff, 0x3cff, 0x3bff, 0x3aff, 
                   0x39ff, 0x38ff, 0x37ff, 0x36ff, 0x35ff, 0x34ff, 0x33ff, 
                   0x32ff, 0x31ff, 0x30ff, 0x2fff, 0x2eff, 0x2dff, 0x2cff, 
                   0x2bff, 0x2aff, 0x29ff, 0x28ff, 0x27ff, 0x26ff, 0x25ff, 
                   0x24ff, 0x23ff, 0x22ff, 0x21ff, 0x20ff, 0x1fff, 0x1eff, 
                   0x1dff, 0x1cff, 0x1bff, 0x1aff, 0x19ff, 0x18ff, 0x17ff, 
                   0x16ff, 0x15ff, 0x14ff, 0x13ff, 0x12ff, 0x11ff, 0x10ff, 
                   0x0fff, 0x0eff, 0x0dff, 0x0cff, 0x0bff, 0x0aff, 0x09ff, 
                   0x08ff, 0x07ff, 0x06ff, 0x05ff, 0x04ff, 0x03ff, 0x02ff, 
                   0x01ff, 0x00ff, 0xffff, 0xfeff, 0xfdff, 0xfcff, 0xfbff, 
                   0xfaff, 0xf9ff, 0xf8ff, 0xf7ff, 0xf6ff, 0xf5ff, 0xf4ff, 
                   0xf3ff, 0xf2ff, 0xf1ff, 0xf0ff, 0xefff, 0xeeff, 0xedff, 
                   0xecff, 0xebff, 0xeaff, 0xe9ff, 0xe8ff, 0xe7ff, 0xe6ff, 
                   0xe5ff, 0xe4ff, 0xe3ff, 0xe2ff, 0xe1ff, 0xe0ff, 0xdfff, 
                   0xdeff, 0xddff, 0xdcff, 0xdbff, 0xdaff, 0xd9ff, 0xd8ff, 
                   0xd7ff, 0xd6ff, 0xd5ff, 0xd4ff, 0xd3ff, 0xd2ff, 0xd1ff, 
                   0xd0ff, 0xcfff, 0xceff, 0xcdff, 0xccff, 0xcbff, 0xcaff, 
                   0xc9ff, 0xc8ff, 0xc7ff, 0xc6ff, 0xc5ff, 0xc4ff, 0xc3ff, 
                   0xc2ff, 0xc1ff, 0xc0ff, 0xbfff, 0xbeff, 0xbdff, 0xbcff, 
                   0xbbff, 0xbaff, 0xb9ff, 0xb8ff, 0xb7ff, 0xb6ff, 0xb5ff, 
                   0xb4ff, 0xb3ff, 0xb2ff, 0xb1ff, 0xb0ff, 0xafff, 0xaeff, 
                   0xadff, 0xacff, 0xabff, 0xaaff, 0xa9ff, 0xa8ff, 0xa7ff, 
                   0xa6ff, 0xa5ff, 0xa4ff, 0xa3ff, 0xa2ff, 0xa1ff, 0xa0ff, 
                   0x9fff, 0x9eff, 0x9dff, 0x9cff, 0x9bff, 0x9aff, 0x99ff, 
                   0x98ff, 0x97ff, 0x96ff, 0x95ff, 0x94ff, 0x93ff, 0x92ff, 
                   0x91ff, 0x90ff, 0x8fff, 0x8eff, 0x8dff, 0x8cff, 0x8bff, 
                   0x8aff, 0x89ff, 0x88ff, 0x87ff, 0x86ff, 0x85ff, 0x84ff, 
                   0x83ff, 0x82ff, 0x81ff, 0x80ff, 0x7fff, 0x7eff, 0x7dff, 
                   0x7cff, 0x7bff, 0x7aff, 0x79ff, 0x78ff, 0x77ff, 0x76ff, 
                   0x75ff, 0x74ff, 0x73ff, 0x72ff, 0x71ff, 0x70ff, 0x6fff, 
                   0x6eff, 0x6dff, 0x6cff, 0x6bff, 0x6aff, 0x69ff, 0x68ff, 
                   0x67ff, 0x66ff, 0x65ff, 0x64ff, 0x63ff, 0x62ff, 0x61ff, 
                   0x60ff, 0x5fff, 0x5eff, 0x5dff, 0x5cff, 0x5bff, 0x5aff, 
                   0x59ff, 0x58ff, 0x57ff, 0x56ff, 0x55ff, 0x54ff, 0x53ff, 
                   0x52ff, 0x51ff, 0x50ff };

/** Precalculated ICMP sequences */
int sequence[] = { 0x0100, 0x0200, 0x0300, 0x0400, 0x0500, 0x0600, 0x0700, 
                   0x0800, 0x0900, 0x0a00, 0x0b00, 0x0c00, 0x0d00, 0x0e00, 
                   0x0f00, 0x1000, 0x1100, 0x1200, 0x1300, 0x1400, 0x1500, 
                   0x1600, 0x1700, 0x1800, 0x1900, 0x1a00, 0x1b00, 0x1c00, 
                   0x1d00, 0x1e00, 0x1f00, 0x2000, 0x2100, 0x2200, 0x2300, 
                   0x2400, 0x2500, 0x2600, 0x2700, 0x2800, 0x2900, 0x2a00, 
                   0x2b00, 0x2c00, 0x2d00, 0x2e00, 0x2f00, 0x3000, 0x3100, 
                   0x3200, 0x3300, 0x3400, 0x3500, 0x3600, 0x3700, 0x3800, 
                   0x3900, 0x3a00, 0x3b00, 0x3c00, 0x3d00, 0x3e00, 0x3f00, 
                   0x4000, 0x4100, 0x4200, 0x4300, 0x4400, 0x4500, 0x4600, 
                   0x4700, 0x4800, 0x4900, 0x4a00, 0x4b00, 0x4c00, 0x4d00, 
                   0x4e00, 0x4f00, 0x5000, 0x5100, 0x5200, 0x5300, 0x5400, 
                   0x5500, 0x5600, 0x5700, 0x5800, 0x5900, 0x5a00, 0x5b00, 
                   0x5c00, 0x5d00, 0x5e00, 0x5f00, 0x6000, 0x6100, 0x6200, 
                   0x6300, 0x6400, 0x6500, 0x6600, 0x6700, 0x6800, 0x6900, 
                   0x6a00, 0x6b00, 0x6c00, 0x6d00, 0x6e00, 0x6f00, 0x7000, 
                   0x7100, 0x7200, 0x7300, 0x7400, 0x7500, 0x7600, 0x7700, 
                   0x7800, 0x7900, 0x7a00, 0x7b00, 0x7c00, 0x7d00, 0x7e00, 
                   0x7f00, 0x8000, 0x8100, 0x8200, 0x8300, 0x8400, 0x8500, 
                   0x8600, 0x8700, 0x8800, 0x8900, 0x8a00, 0x8b00, 0x8c00, 
                   0x8d00, 0x8e00, 0x8f00, 0x9000, 0x9100, 0x9200, 0x9300, 
                   0x9400, 0x9500, 0x9600, 0x9700, 0x9800, 0x9900, 0x9a00, 
                   0x9b00, 0x9c00, 0x9d00, 0x9e00, 0x9f00, 0xa000, 0xa100, 
                   0xa200, 0xa300, 0xa400, 0xa500, 0xa600, 0xa700, 0xa800, 
                   0xa900, 0xaa00, 0xab00, 0xac00, 0xad00, 0xae00, 0xaf00, 
                   0xb000, 0xb100, 0xb200, 0xb300, 0xb400, 0xb500, 0xb600, 
                   0xb700, 0xb800, 0xb900, 0xba00, 0xbb00, 0xbc00, 0xbd00, 
                   0xbe00, 0xbf00, 0xc000, 0xc100, 0xc200, 0xc300, 0xc400, 
                   0xc500, 0xc600, 0xc700, 0xc800, 0xc900, 0xca00, 0xcb00, 
                   0xcc00, 0xcd00, 0xce00, 0xcf00, 0xd000, 0xd100, 0xd200, 
                   0xd300, 0xd400, 0xd500, 0xd600, 0xd700, 0xd800, 0xd900, 
                   0xda00, 0xdb00, 0xdc00, 0xdd00, 0xde00, 0xdf00, 0xe000, 
                   0xe100, 0xe200, 0xe300, 0xe400, 0xe500, 0xe600, 0xe700, 
                   0xe800, 0xe900, 0xea00, 0xeb00, 0xec00, 0xed00, 0xee00, 
                   0xef00, 0xf000, 0xf100, 0xf200, 0xf300, 0xf400, 0xf500, 
                   0xf600, 0xf700, 0xf800, 0xf900, 0xfa00, 0xfb00, 0xfc00, 
                   0xfd00, 0xfe00, 0xff00 };

